// -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
//
// pdftoc, version 1.0
// visit: www.pdfhacks.com/pdftoc/
//

#include <cstdio>
#include <cstring>
#include <string>
#include <map>
#include <fstream>
#include <iostream>

using namespace std;

static int
append_toc( char* pdf_fn, char* target )
  // take bookmark data from cin
  // and create nested lists of HTML hyperlinks
  //
  // target is the toc page hyperlink frame target name
{
  int curr_level= 1;

  while( cin ) {
    string name;
    string title;
    int level= 1;
    int page_num= 1;

    // cue burst data up to next bookmark and grab title
    while( cin ) {
      cin >> name;
      if( name== "BookmarkTitle:" ) {
        cin.get(); // drop init space
        getline( cin, title );
        break;
      }
    }
    if( cin ) { // the bookmark level
      cin >> name >> level;
      if( name!= "BookmarkLevel:" ) { // error
        cerr << "Error: Unexpected matter in burst data file; expected bookmark level." << endl;
        break;
      }
    }
    else { // end
      break;
    }
    if( cin ) { // the bookmark page reference
      cin >> name >> page_num;
      if( name!= "BookmarkPageNumber:" ) { // error
        cerr << "Error: Unexpected matter in burst data file; expected bookmark page number." << endl;
        break;
      }
    }
    else { // error
      cerr << "Error: Unexpected end of burst data file." << endl;
      break;
    }

    // list tags, if any; note that PDF bookmarks don't
    // skip heading levels, they increase or decrease one at a time;
    //
    if( level== curr_level ) { // same level
      if( level== 1 ) { // top-most level isn't in a list
        cout << "\n";
      }
      else {
        cout << "</li>\n<li>";
      }
    }
    else if( curr_level< level ) { // higher level
      cout << "\n<ul><li>";
    }
    else { // lower level
      cout << "</li></ul>";

      if( level== 1 ) {
        cout << "\n";
      }
      else {
        cout << "</li>\n<li>";
      }
    }

    if( target ) {
      char buff[256];
      sprintf( buff, 
               "<div class=\"li-%d\"><a href=\"%s#page=%d\" target=\"%s\">", 
               level- 1,
               pdf_fn,
               page_num,
               target );

      cout << buff;
    }
    else { // no target
      char buff[256];
      sprintf( buff, 
               "<div class=\"li-%d\"><a href=\"%s#page=%d\">", 
               level- 1,
               pdf_fn,
               page_num );

      cout << buff;
    }
    cout << title;
    cout << "</a></div>";

    curr_level= level;
  }

  // close
  while( 1<= curr_level-- ) {
    cout << "</li></ul>";
  }

  return 0;
}

static void version();
static void help();

int main( int argc, char** argv )
{
  { // help? version?
    bool help_b= (argc!= 2);
    bool version_b= false;
    for( int ii= 1; ii< argc; ++ii ) {
      help_b= help_b || ( strcmp( argv[ii], "--help" )== 0 );
      version_b= version_b || ( strcmp( argv[ii], "--version" )== 0 );
    }
    if( help_b ) {
      version();
      help();
      return 0;
    }
    else if( version_b ) {
      version();
      return 0;
    }      
  }

  return append_toc( argv[1], 0 );

}

static void version()
{
	cout << endl;
  cout << "pdftoc 1.00, Link your PDF with the Web" << endl;
	cout << "Copyright (C) 2003, Sid Steward - Visit: www.pdfhacks.com/pdftoc/" << endl;
	cout << "This is free software; see the source code for copying conditions. There is" << endl;
	cout << "NO warranty, not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." << endl;
	cout << endl;
}

static void help()
{
  cout << "Usage:" << endl;
  cout << "   pdftoc <PDF path and filename>" << endl;
  cout << endl;
  cout << "Where:" << endl;
  cout << "   pdftoc takes the output from a pdftk dump_data operation or the" << endl;
  cout << "      doc_data.txt file from a pdftk burst operation on stdin.  It" << endl;
  cout << "      creates a hyperlinked, HTML Table of Contents and dumps it" << endl;
  cout << "      to stdout.  So, you might call it like this:" << endl;
  cout << "         pdftoc fulldoc.pdf < doc_data.txt > doc_toc.html" << endl;
  cout << "      or possibly in conjunciton with pdftk:" << endl;
  cout << "         pdftk fulldoc.pdf dump_data | pdftoc fulldoc.pdf > doc_toc.html" << endl;
  cout << endl;
  cout << "   <PDF path and filename>, if given, is simply the (possibly relative)" << endl;
  cout << "      URL to the PDF for which we are building the Table of Contents." << endl;
}

